/*
# Copyright (c) 2003-2004 by the University of Southern California.
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation in source and binary forms for non-commercial purposes and
# without fee is hereby granted, provided that the above copyright notice
# appear in all copies and that both the copyright notice and this permission
# notice appear in supporting documentation, and that any documentation,
# advertising materials, and other materials related to such distribution and
# use acknowledge that the software was developed by the University of
# Southern California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived from this
# software without specific prior written permission.
#
# THE UNIVERSITY OF SOUTHERN CALIFORNIA MAKES NO REPRESENTATIONS ABOUT THE
# SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS PROVIDED "AS
# IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT
# LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE.
#
# Other copyrights might apply to parts of this software and are so noted when
# applicable.
# ----------------------------------------------------------------------------
#
# Primary author: Purushotham Kamath
#                 Sept  2004
#This material is based upon work supported by the Defense Advanced Research 
#Projects Agency under contract no. N66001-02-1-8939 issued by the Space and 
#Naval Warfare Systems Center (SPAWAR).  Any opinions, findings, and conclusions 
#or recommendations expressed in this material are those of the author(s) and 
#do not necessarily reflect the views of the Defense Advanced Research Projects
#Agency, SPAWAR, or the U.S. Government.
*/


#ifndef PACKET_H
#define PACKET_H
#include <rng.h>

//Abstraction of a packet.
//Contains some fields usually present in link layer, network layer and transport layer headers
//Contains physical layer transmission parameters (codeset paramters)
//Also contains fields needed for simulation (arrival time of packet, time of departure, location of departure)
//Also contains statistics (all layers)


//The LineCodeword class is used for Distributed Perfect Scheduling
//It stores a list of codewords on the line. 
//Not used by any other scheduling mechanism.
class LineCodeword
{
public:
	LineCodeword(){ next = NULL; }
	~LineCodeword(){ delete codeword;}
	LineCodeword * next;
	int * codeword;
};



//Packet state
enum{PACKET_WAITINGFORTX=0, PACKET_INTRANSIT, PACKET_DEFERED, PACKET_OK, PACKET_ERROR_MUI, PACKET_DROPPED};

//Modulation type
enum{OOK= 0, CCM};


class Packet 
{
private: 
public:

//List data members
	Packet *next;
	Packet *prev;

//Packet header
	int id;      //Packet identification.
	int modulation; //Type of modulation

//Link/Network/Transport layer fields 
	int srcaddr; //Source address.
	int srcport; //Source port.
	int dstaddr; //Destination address.
	int dstport; //Destination port.
	int bytesize;   //Total size of packets (all headers + data) in bytes
	int symsize;    //Total size of packets (all headers + data) in symbols
			//For OOK, number of symbols = number of bits
			//For CCM, number of symbols = number of bits/log(codeset length) (to the base 2)

//Physical layer transmsision parameters
	double bitrate;
	double chiprate;
	
	int codeword;		//index into codeset array
	int A;
	int w;
	int N;

//Departure point (Point in space where transmission of the packet began).
	double depx;

//Arrival time (Time when the packet arrived at the interface of the node for transmission).
	double arrivaltime;
//Departure time (Time when the transmission of the packet (first bit) began).
	double departuretime;
	int offset;

//Packet state
	int status;
	int backoffs;   //Number of attempts before the packet was transmittted

//Used to store state estimate
	int *state;
	int *data;
	double ep;

//Random number generator for data
	RNG * datarng;

//Number of defers in tx scheduling module 
	int defers;

//Codewords on the line - used for DPS
	LineCodeword * linecodewords;	

	Packet();
	Packet(Packet & p);
	void calcboundary(double *, double *);
	int *calcstate(double x, int allones );
	void initialize();
	virtual ~Packet();
	int val(int, int);
	void checkdata();

protected:


};



#endif

