/*
# Copyright (c) 2003-2004 by the University of Southern California.
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation in source and binary forms for non-commercial purposes and
# without fee is hereby granted, provided that the above copyright notice
# appear in all copies and that both the copyright notice and this permission
# notice appear in supporting documentation, and that any documentation,
# advertising materials, and other materials related to such distribution and
# use acknowledge that the software was developed by the University of
# Southern California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived from this
# software without specific prior written permission.
#
# THE UNIVERSITY OF SOUTHERN CALIFORNIA MAKES NO REPRESENTATIONS ABOUT THE
# SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS PROVIDED "AS
# IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT
# LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE.
#
# Other copyrights might apply to parts of this software and are so noted when
# applicable.
# ----------------------------------------------------------------------------
#
# Primary author: Purushotham Kamath
#                 Sept  2004
#This material is based upon work supported by the Defense Advanced Research 
#Projects Agency under contract no. N66001-02-1-8939 issued by the Space and 
#Naval Warfare Systems Center (SPAWAR).  Any opinions, findings, and conclusions 
#or recommendations expressed in this material are those of the author(s) and 
#do not necessarily reflect the views of the Defense Advanced Research Projects
#Agency, SPAWAR, or the U.S. Government.
*/

#include <stdlib.h>
#include <stdarg.h>

#include <packet.h>
#include <eventhandler.h>
#include <queue.h>
#include <debug.h>

//Queue class functions.
//Implements a queue of packets.
//Normal operation is FIFO, however it is possible to do selective dequeuing of a packet from the queue.

Queue::Queue(int s): size(s *1024), bufferoverflows(0), byteoverflows(0)
{
	qsize =0;
	avgqsize =0;
	maxqsize =0;
	pktcnt =0;
	head = NULL;
	tail = NULL;	
	bytes =0;
}

Queue::~Queue()
{

}

int Queue:: enqueue(Packet *pkt)
{

	//dprintf("Queue.cc: Queue::enqueue(): Entered function\n");	


	if (tail == NULL)
	{

//Kamath: 07/09/01: Bug fix
//Check the queue size limitation for the case when there are no packets in 
//the queue
		if((size != 0) && (pkt->bytesize > size))
		{
			dprintf("Queue.cc: Queue::enqueue(): Buffer overflow.\n");	
			bufferoverflows++;
			byteoverflows+=pkt->bytesize;
			return 0;
		}

		qsize++;
		bytes+=pkt->bytesize;

		//dprintf("Queue.cc: Queue::enqueue(): No packet in queue.\n");	

		if(qsize > maxqsize)
			maxqsize=qsize;	


		head = pkt;
		tail = pkt;
		pkt->next = NULL;
		pkt->prev = NULL;
		return 1;
	}

	Packet *p = tail;
	if((bytes+ pkt->bytesize > size) && (size != 0))
	{
		dprintf("Queue.cc: Queue::enqueue(): Buffer overflow.\n");	
		byteoverflows+=pkt->bytesize;
		bufferoverflows++;
		return 0;
	}


	tail = pkt;
	p->next = pkt;
	pkt->prev = p;
	pkt->next = NULL;
	qsize++;
	bytes+=pkt->bytesize;
	return 1;
}

Packet * Queue:: dequeue()
{
	dprintf("Queue.cc: Queue::dequeue(): Entered function\n");	
	if(head != NULL)
	{
		qsize--;
		Packet *p = head;
		bytes-=p->bytesize;
		head = head->next;
		if (head == NULL)
			tail = NULL;
		dprintf("Queue.cc: Queue::dequeue(): Leaving function\n");	
		return p;
	}
	else 
	{
		dprintf("Queue.cc: Queue::dequeue(): Nothing in queue: Leaving function\n");	
		return NULL;
	}
}


//Dequeue  a specific packet. Searches for the packet based on the 
//pointer to the packet. 
//TBD: Change this to a search based on packet id.

Packet * Queue:: dequeue(Packet *rp)
{
	dprintf("Queue.cc: Queue::dequeue2(): Entered function\n");	
	if(head != NULL)
	{
		Packet *p = head;
		while(p != rp)
		{
			if(p  ==NULL)
				return NULL;
			p=p->next;
		}
	
		if (p== NULL)
		{
			dprintf("Queue.cc: Queue::dequeue2(): Not found\n");	
			return NULL;
		}
		
		qsize--;
		bytes-=p->bytesize;

		dprintf("Queue.cc: Queue::dequeue2(): Head\n");	
		if(p == head)
		{
			head = p->next;
			if (tail == p)
				tail = p->next;
			return p;
		}

		dprintf("Queue.cc: Queue::dequeue2(): Tail\n");	
		if(p == tail)
		{
			tail = p->prev;
			tail->next  = NULL;
			return p;
		}

		dprintf("Queue.cc: Queue::dequeue2(): Neither\n");	
		p->prev->next  = p->next;
		p->next->prev = p->prev;

		dprintf("Queue.cc: Queue::dequeue(): Leaving function\n");	
		return p;
	}
	else 
	{
		dprintf("Queue.cc: Queue::dequeue(): Nothing in queue: Leaving function\n");	
		return NULL;
	}
}




void Queue::dumpstats()
{
	printf("Queue.cc: Queue::dumpstats(): Number of bufferoverflows: %d\n", bufferoverflows);
	printf("Queue.cc: Queue::dumpstats(): Byte overflows: %d\n", byteoverflows);
}
