/*
# Copyright (c) 2003-2004 by the University of Southern California.
# All rights reserved.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation in source and binary forms for non-commercial purposes and
# without fee is hereby granted, provided that the above copyright notice
# appear in all copies and that both the copyright notice and this permission
# notice appear in supporting documentation, and that any documentation,
# advertising materials, and other materials related to such distribution and
# use acknowledge that the software was developed by the University of
# Southern California, Information Sciences Institute.  The name of the
# University may not be used to endorse or promote products derived from this
# software without specific prior written permission.
#
# THE UNIVERSITY OF SOUTHERN CALIFORNIA MAKES NO REPRESENTATIONS ABOUT THE
# SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS PROVIDED "AS
# IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT
# LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
# PARTICULAR PURPOSE.
#
# Other copyrights might apply to parts of this software and are so noted when
# applicable.
# ----------------------------------------------------------------------------
#
# Primary author: Purushotham Kamath
#                 Sept  2004
#This material is based upon work supported by the Defense Advanced Research 
#Projects Agency under contract no. N66001-02-1-8939 issued by the Space and 
#Naval Warfare Systems Center (SPAWAR).  Any opinions, findings, and conclusions 
#or recommendations expressed in this material are those of the author(s) and 
#do not necessarily reflect the views of the Defense Advanced Research Projects
#Agency, SPAWAR, or the U.S. Government.
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

#include <packet.h>
#include <eventhandler.h>
#include <queue.h>
#include <scheduler.h>
#include <state.h>
#include <rng.h>
#include <trgen.h>
#include <trdump.h>
#include <txsched.h>
#include <intransit.h>
#include <codeset.h>
#include <debug.h>
#include <receiver.h>
#include <stateest.h>
#include <transmitter.h>
#include <state.h>
#include <statshandler.h>


extern Scheduler * sched;
extern InTransit * it;
extern TrDump * trdump;
extern TrGen * trgen;
extern char outputfile[20];
extern Codeset *codeset;
extern Receiver *rcvr;
extern StateEstimator *stateest;
extern TxSched *txsched;
extern StatsHandler *stats;
extern int propSpeed;


//The transmitter class. It implemnts the functionality of
//the transitter.
//On packet arrival it estimates the state (using the state estimator)
//and schedules the transmission (using the transmission scheduler).

Transmitter::Transmitter():EventHandler()
{
	debug = debugon("transmitter");;
}


//The event handling function of the Transmitter. It dequeues a packet and
//sends it to the next component.
void Transmitter::handleevent(Packet *p)
{
	dtprintf("transmitter.cc: Transmitter::handleevent(): Entered function: %d\n", p->id);
	stateest->process(p);
}



void Transmitter::dumpstats()
{
}



Transmitter::~Transmitter()
{
}


void Transmitter::process(Packet *p)
{

	dtprintf("transmitter.cc: Transmitter::process(): Scheduling packet: %d\n", p->id);
	//Estimate the state
	sched->scheduleevent(this, p, 0 );
}
